package main

import (
	"fmt"
	"log"
	"os"
	"strconv"
)

const (
	MaxInt = int(^uint(0) >> 1)
	MinInt = -MaxInt - 1
)

var timeMachine *TimeMachine

func main() {
	if len(os.Args) < 2 {
		fmt.Println("Usage: day11 <serial #>")
		os.Exit(1)
	}
	timeMachine = &TimeMachine{
		serial:     Atoi(os.Args[1]),
		fuelLevels: make(map[string]int),
	}
	fmt.Println(timeMachine.getSquareFuelLevel(283, 169, 13))
	os.Exit(0)
	highest := MinInt
	highestX, highestY, highestSize := 0, 0, 1
	for size := 1; size <= 13; size++ {
		fmt.Println("=", size, "=")
		for i := 0; i < 300-size; i++ {
			for j := 0; j < 300-size; j++ {
				w := timeMachine.getSquareFuelLevel(i, j, size)
				if w > highest {
					highestX, highestY = i, j
					highestSize = size
					highest = w
					fmt.Printf("New Highest: %d,%d,%d : %d\n", highestX, highestY, highestSize, highest)
				}
			}
		}
	}
	fmt.Printf("Highest: %d,%d,%d : %d\n", highestX, highestY, highestSize, highest)
}

type TimeMachine struct {
	serial     int
	fuelLevels map[string]int
}

func (t *TimeMachine) getKeyForSquare(x, y, size int) string {
	bx := strconv.Itoa(x)
	by := strconv.Itoa(y)
	bs := strconv.Itoa(size)
	return bx + "," + by + "," + bs
}

func (t *TimeMachine) getSquareFuelLevel(x, y, size int) int {
	k := t.getKeyForSquare(x, y, size)
	if v, ok := t.fuelLevels[k]; ok {
		return v
	}
	var ret int
	if size == 1 {
		ret = t.calcFuelLevel(x, y)
	} else {
		ret = t.getSquareFuelLevel(x, y, size-1)
		// Add the right-most cells
		for i := 0; i < size; i++ {
			ret += t.calcFuelLevel(x+(size-1), y+i)
		}
		// Add the bottom cells
		for i := 0; i < size-1; i++ {
			ret += t.calcFuelLevel(x+i, y+(size-1))
		}
	}
	t.fuelLevels[k] = ret
	return ret
}

// calcFuelLevel calculates one cells fuel level
func (t *TimeMachine) calcFuelLevel(x, y int) int {
	rackId := x + 10
	startingLevel := rackId * y
	currLevel := startingLevel + t.serial
	currLevel *= rackId
	var ret int
	if currLevel < 100 {
		ret = -5
	} else {
		ret = ((currLevel / 100) % 10) - 5
	}
	return ret
}

// calcSquareFuelLevel calculates all cells from x, y -> x+size, y+size
func (t *TimeMachine) calcSquareFuelLevel(x, y, size int) int {
	var ret int
	for i := 0; i < size; i++ {
		for j := 0; j < size; j++ {
			ret += t.calcFuelLevel(x+i, y+j)
		}
	}
	return ret
}

func Atoi(i string) int {
	var ret int
	var err error
	if ret, err = strconv.Atoi(i); err != nil {
		log.Fatal("Invalid Atoi")
	}
	return ret
}
